using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace Foosball
{
    public class FoosMan : FoosObj
    {
        public const float ARM_LENGTH = 10.0F;
        public const int COLLISION_COOLDOWN = 10;

        public enum Wall
        {
            LEFT_WALL = 0,
            RIGHT_WALL,
            UP_WALL,
            DOWN_WALL,
            NO_INDEX_WALL,
        };        

        Rod _rod;                
        protected BoundingSphere _colSphere;
        protected CollisionFace[] _colFaces;

        protected Vector3 _speed, _prevPos, _curPos;
        protected Vector3 _relativeToRod;
        protected int _collided;

        public int Collided
        {
            get
            {
                return _collided;
            }
            set
            {
                _collided = value;
            }
        }

        public Vector3 CurrentPos
        {
            get
            {
                return _rod.position + _relativeToRod + 
                    Vector3.Transform(_colSphere.Center, 
                    Matrix.CreateRotationZ(_rod.zRotation)); 
            }
        }
        
        public FoosMan(Model model, Rod rod, Vector3 relativeToRod)
            : base(model)
        {
            _collided = 0;
            _rod = rod;
            _relativeToRod = relativeToRod;
            _pos.Y = FoosGame.ROD_TO_SURF;
                                    
            foreach (ModelMesh mesh in model.Meshes)
            {
                if (mesh.Name.Equals("tail"))
                {                    
                    _colSphere.Center = new Vector3(0.0F, -ARM_LENGTH, 0.0F);
                    _colSphere.Radius = 2.4F;
                }
            }

            _curPos = CurrentPos;
        }

        public override void Update(Control control, double milliseconds)
        {
            // Notice: rod should call Update first
            _prevPos = _curPos;

            if (_rod.rotationSpeed != 0 || _rod.zMovSpeed != 0)                            
                _curPos = CurrentPos;            

            _speed = (_curPos - _prevPos) / (float)milliseconds;
        }


        /// <summary>
        /// Static methods for ball-men collision detection
        /// </summary>
        /// <param name="allObjs"></param>
        protected enum ColType { COL_NONE, COL_WALL, COL_MAN };

        protected static ColType colType;
        protected static float ratioElapsed, tempElapsed;
        protected static Wall colWall;
        protected static FoosObj colObj;
        protected static Vector3 colPoint, newDir;
        protected static Vector3[] edge = {Vector3.Zero, Vector3.Zero};
        protected static int[] _collidedWall = { 0, 0, 0, 0};
        

        protected static void collisionEdge(Vector3 ballOldPos, Vector3 ballRelPos, float restElapsed)
        {
            Vector3 c, ballNewPos = ballOldPos + ballRelPos;
            float ballRadius = FoosGame.BALL_RADIUS;

            for (int i = 0; i < 2; i++)
            {
                Vector3 o = ballOldPos - edge[i];
                float ol = o.Length();
                Debug.Assert(ol >= ballRadius); // Hopefully:)

                Vector3 u = ballRelPos;

                // ball's relative end position to man
                Vector3 e = o + u;

                // Now get the length of relative displacement and its normal
                float ul = u.Length();
                u.Normalize();


                float t = -Vector3.Dot(o, u);
                Vector3 s;  // s stores the shortest vector from origion to ball
                if (t >= ul)
                    s = e;
                else if (t <= 0)
                    s = o;
                else
                    s = o + t * u;


                // If we collide
                float sl = s.Length();
                if (sl < ballRadius)
                {
                    // Find the collision point
                    float ol2 = o.LengthSquared();
                    //float dist_s2 = ballRadius * ballRadius;

                    float delta = (float)Math.Sqrt((double)(t * t + ballRadius * ballRadius - ol2));

                    float t1 = t - delta;
                    // if (t1 < 1e-5F)
                    //     t1 = 0;
                    Debug.Assert(t1 <= ul && t1 >= 0);

                    // Now we got t1
                    float ratio = t1 / ul;

                    // c is collision vector in man's coordinate system
                    // cu is the collision normal vector   

                    c = o + t1 * u;
                    while (c.Length() < ballRadius + 0.001 && t1 > 0)
                    {
                        if (t1 < 1e-3F)
                            t1 = 0;
                        else
                            t1 -= 1e-3F;
                        c = o + t1 * u;
                    }
                    ratio = t1 / ul;

                    if (ratio < ratioElapsed)
                    {
                        colType = ColType.COL_WALL;
                        
                        ratioElapsed = ratio;
                        tempElapsed = ratio * restElapsed;
                        colPoint = c + edge[i];

                        Vector3 cu = c;
                        float cul = c.Length();
                        cu.Normalize();

                        Vector3 c2 = Vector3.Dot(o, cu) * cu - o;
                        newDir = c2 + c2 + o - c;
                        if (newDir.Length() < 1e-6)
                            newDir = Vector3.Zero;
                        else
                            newDir.Normalize();
                    }
                }
            }
        }

        public static void collisionDetection(FoosObj[] allObjs, double elapsedMilliseconds, FoosGame game)
        {
            float ll;
            ColType colLast;
            

            // ****************************************************************************
            // Call this function after calling each man's Update()
            // ****************************************************************************
            
            FBall ball = (FBall)allObjs[FoosGame.OI_BALL];
            if (ball.positionX >= FoosGame.TABLE_LENGTH * 0.5F - FoosGame.BALL_RADIUS
                || ball.positionX <= -FoosGame.TABLE_LENGTH * 0.5F + FoosGame.BALL_RADIUS
                || ball.positionY < -29.0F)
            {
                // Waiting for new set starting
                ball.position = new Vector3(0, -30.0F, 0);
                ball.Speed = Vector3.Zero;                
            }

            FoosMan[] oBMen = new FoosMan[FoosGame.BLACKMEN_NUM + FoosGame.YELLOWMEN_NUM];

            for (int i = 0; i < FoosGame.BLACKMEN_NUM; i++)
            {
                oBMen[i] = (FoosMan)allObjs[FoosGame.OI_BLACKMEN + i];
                if (oBMen[i].Collided >= COLLISION_COOLDOWN)
                    oBMen[i].Collided = 0;
                else if (oBMen[i].Collided > 0)
                    oBMen[i].Collided++;                
            }

            for (int i = 0; i < FoosGame.BLACKMEN_NUM; i++)
            {
                oBMen[i + FoosGame.BLACKMEN_NUM] = (FoosMan)allObjs[FoosGame.OI_YELLOWMEN + i];
                if (oBMen[i + FoosGame.BLACKMEN_NUM].Collided >= COLLISION_COOLDOWN)
                    oBMen[i + FoosGame.BLACKMEN_NUM].Collided = 0;
                else if (oBMen[i + FoosGame.BLACKMEN_NUM].Collided > 0)
                    oBMen[i + FoosGame.BLACKMEN_NUM].Collided++;    
            }

            for (int i = 0;i < (int)Wall.NO_INDEX_WALL;i++)
            {
                if (_collidedWall[i] >= COLLISION_COOLDOWN)
                    _collidedWall[i] = 0;
                else if (_collidedWall[i] > 0)
                    _collidedWall[i]++;
            }
                
            float halfTableL = FoosGame.FIELD_LENGTH * 0.5F;
            float halfTableW = FoosGame.FIELD_WIDTH * 0.5F;
            float ballRadius = FoosGame.BALL_RADIUS;

            float restElapsed = (float)elapsedMilliseconds;
            int contiZero = 0;
            while (restElapsed > 0)
            {
                // ball's old position
                Vector3 ballOldPos = ball.position;
                // ball's displacement in elapsedMilliseconds
                Vector3 ballRelPos = ball.Speed * restElapsed;


                colType = ColType.COL_NONE;
                ratioElapsed = 1.0F;
                tempElapsed = restElapsed;
                colWall = Wall.NO_INDEX_WALL;
                colObj = null;
                colPoint = Vector3.Zero;
                newDir = Vector3.Zero;
                
                Vector3 c;

                // Collide ball with walls
                Vector3 ballNewPos = ballOldPos + ballRelPos;
//                if (ballOldPos.X <= -ballRadius - halfTableL && !FoosGame.NewSet)
                if (ballOldPos.X <= -103 && !FoosGame.NewSet)
                {
                    FoosGame.YellowWin = FoosGame.YellowWin + 1;
                    FoosGame.NewSet = true;
                    FoosGame.CoolDown = FoosGame.COOL_DOWN;
                }

              //  if (ballOldPos.X >= ballRadius + halfTableL && !FoosGame.NewSet)
                if (ballOldPos.X >= 103 && !FoosGame.NewSet)
                {
                    FoosGame.BlackWin = FoosGame.BlackWin + 1;
                    FoosGame.NewSet = true;
                    FoosGame.CoolDown = FoosGame.COOL_DOWN;
                }

                // Inside Left wall
                if (ballOldPos.X < ballRadius - halfTableL)
                {
                    if (ballOldPos.Z >= -FoosGame.HALF_GATE_WIDTH + ballRadius
                    && ballNewPos.Z < -FoosGame.HALF_GATE_WIDTH + ballRadius)
                    {
                        float ratio = (-FoosGame.HALF_GATE_WIDTH + ballRadius- ballOldPos.Z)
                            / (ballNewPos.Z - ballOldPos.Z);

                        c = ballOldPos + ratio * ballRelPos;
                        while (c.Z < 0.001 - FoosGame.HALF_GATE_WIDTH + ballRadius&& ratio > 0)
                        {
                            if (ratio < 1e-3)
                                ratio = 0;
                            else
                                ratio -= 1e-3F;
                            c = ballOldPos + ratio * ballRelPos;
                        }

                        if (ratio < ratioElapsed)
                        {
                            colType = ColType.COL_WALL;

                            ratioElapsed = ratio;
                            tempElapsed = ratio * restElapsed;
                            colPoint = c;

                            newDir = ballNewPos - ballOldPos;
                            newDir.Z = -newDir.Z;
                            newDir.Normalize();
                        }
                    }

                    if (ballOldPos.Z <= FoosGame.HALF_GATE_WIDTH - ballRadius
                    && ballNewPos.Z > FoosGame.HALF_GATE_WIDTH - ballRadius)
                    {
                        float ratio = (FoosGame.HALF_GATE_WIDTH - ballRadius - ballOldPos.Z)
                            / (ballNewPos.Z - ballOldPos.Z);

                        c = ballOldPos + ratio * ballRelPos;
                        while (c.Z > - 0.001 + FoosGame.HALF_GATE_WIDTH - ballRadius && ratio > 0)
                        {
                            if (ratio < 1e-3)
                                ratio = 0;
                            else
                                ratio -= 1e-3F;
                            c = ballOldPos + ratio * ballRelPos;
                        }

                        if (ratio < ratioElapsed)
                        {
                            colType = ColType.COL_WALL;

                            ratioElapsed = ratio;
                            tempElapsed = ratio * restElapsed;
                            colPoint = c;

                            newDir = ballNewPos - ballOldPos;
                            newDir.Z = -newDir.Z;
                            newDir.Normalize();
                        }
                    }

                    edge[0] = new Vector3(-halfTableL, ball.positionY, FoosGame.HALF_GATE_WIDTH);
                    edge[1] = new Vector3(-halfTableL, ball.positionY, -FoosGame.HALF_GATE_WIDTH);
                    collisionEdge(ballOldPos, ballRelPos, restElapsed);  
                }

               
                // Left wall                
                if (ballOldPos.X >= ballRadius - halfTableL
                    && ballNewPos.X < ballRadius - halfTableL)
                {
                    float ratio = (ballRadius - halfTableL - ballOldPos.X)
                        / (ballNewPos.X - ballOldPos.X);

                    c = ballOldPos + ratio * ballRelPos;
                    while (c.X < 0.001 + ballRadius - halfTableL && ratio > 0)
                    {
                        if (ratio < 1e-3)
                            ratio = 0;
                        else
                            ratio -= 1e-3F;
                        c = ballOldPos + ratio * ballRelPos;
                    }

                    if (c.Z < FoosGame.HALF_GATE_WIDTH && c.Z >= -FoosGame.HALF_GATE_WIDTH)
                    {
                        edge[0] = new Vector3(- halfTableL, ball.positionY, FoosGame.HALF_GATE_WIDTH);
                        edge[1] = new Vector3(- halfTableL, ball.positionY, -FoosGame.HALF_GATE_WIDTH);
                        collisionEdge(ballOldPos, ballRelPos, restElapsed);                                                        
                    } 
                    else if (ratio < ratioElapsed)
                    {                        
                        colType = ColType.COL_WALL;
                        colWall = Wall.LEFT_WALL;
                                                
                        ratioElapsed = ratio;
                        tempElapsed = ratio * restElapsed;
                        colPoint = c;

                        newDir = ballNewPos - ballOldPos;
                        newDir.X = -newDir.X;
                        newDir.Normalize();
                    }
                }

                // Inside right wall
                if (ballOldPos.X > halfTableL - ballRadius)
                {
                    if (ballOldPos.Z >= -FoosGame.HALF_GATE_WIDTH + ballRadius
                    && ballNewPos.Z < -FoosGame.HALF_GATE_WIDTH + ballRadius)
                    {
                        float ratio = (-FoosGame.HALF_GATE_WIDTH + ballRadius - ballOldPos.Z)
                            / (ballNewPos.Z - ballOldPos.Z);

                        c = ballOldPos + ratio * ballRelPos;
                        while (c.Z < 0.001 - FoosGame.HALF_GATE_WIDTH + ballRadius && ratio > 0)
                        {
                            if (ratio < 1e-3)
                                ratio = 0;
                            else
                                ratio -= 1e-3F;
                            c = ballOldPos + ratio * ballRelPos;
                        }

                        if (ratio < ratioElapsed)
                        {
                            colType = ColType.COL_WALL;

                            ratioElapsed = ratio;
                            tempElapsed = ratio * restElapsed;
                            colPoint = c;

                            newDir = ballNewPos - ballOldPos;
                            newDir.Z = -newDir.Z;
                            newDir.Normalize();
                        }
                    }
                    
                    if (ballOldPos.Z <= FoosGame.HALF_GATE_WIDTH - ballRadius
                    && ballNewPos.Z > FoosGame.HALF_GATE_WIDTH - ballRadius)
                    {
                        float ratio = (FoosGame.HALF_GATE_WIDTH - ballRadius - ballOldPos.Z)
                            / (ballNewPos.Z - ballOldPos.Z);

                        c = ballOldPos + ratio * ballRelPos;
                        while (c.Z > - 0.001 + FoosGame.HALF_GATE_WIDTH - ballRadius && ratio > 0)
                        {
                            if (ratio < 1e-3)
                                ratio = 0;
                            else
                                ratio -= 1e-3F;
                            c = ballOldPos + ratio * ballRelPos;
                        }

                        if (ratio < ratioElapsed)
                        {
                            colType = ColType.COL_WALL;

                            ratioElapsed = ratio;
                            tempElapsed = ratio * restElapsed;
                            colPoint = c;

                            newDir = ballNewPos - ballOldPos;
                            newDir.Z = -newDir.Z;
                            newDir.Normalize();
                        }
                    }

                    edge[0] = new Vector3(halfTableL, ball.positionY, FoosGame.HALF_GATE_WIDTH);
                    edge[1] = new Vector3(halfTableL, ball.positionY, -FoosGame.HALF_GATE_WIDTH);
                    collisionEdge(ballOldPos, ballRelPos, restElapsed);
                } 

                // Right wall                
                if (ballOldPos.X <= halfTableL - ballRadius
                    && ballNewPos.X > halfTableL - ballRadius)
                {                   
                    float ratio = (halfTableL - ballRadius - ballOldPos.X)
                        / (ballNewPos.X - ballOldPos.X);

                    c = ballOldPos + ratio * ballRelPos;
                    while (c.X > halfTableL - ballRadius - 0.001 && ratio > 0)
                    {
                        if (ratio < 1e-3)
                            ratio = 0;
                        else
                            ratio -= 1e-3F;
                        c = ballOldPos + ratio * ballRelPos;
                    }

                    if (c.Z < FoosGame.HALF_GATE_WIDTH && c.Z >= -FoosGame.HALF_GATE_WIDTH)
                    {
                        edge[0] = new Vector3(halfTableL, ball.positionY, FoosGame.HALF_GATE_WIDTH);
                        edge[1] = new Vector3(halfTableL, ball.positionY, -FoosGame.HALF_GATE_WIDTH);
                        collisionEdge(ballOldPos, ballRelPos, restElapsed);
                    } 
                    else if (ratio < ratioElapsed)
                    {
                        colType = ColType.COL_WALL;
                        colWall = Wall.RIGHT_WALL;
                                                
                        ratioElapsed = ratio;
                        tempElapsed = ratio * restElapsed;
                        colPoint = c;

                        newDir = ballNewPos - ballOldPos;
                        newDir.X = -newDir.X;
                        newDir.Normalize();
                    }
                }

                // Up wall                
                if (ballOldPos.Z >= ballRadius - halfTableW
                    && ballNewPos.Z < ballRadius - halfTableW)
                {
                    float ratio = (ballRadius - halfTableW - ballOldPos.Z)
                        / (ballNewPos.Z - ballOldPos.Z);

                    c = ballOldPos + ratio * ballRelPos;
                    while (c.Z < ballRadius - halfTableW + 0.001 && ratio > 0)
                    {
                        if (ratio < 1e-3)
                            ratio = 0;
                        else
                            ratio -= 1e-3F;
                        c = ballOldPos + ratio * ballRelPos;
                    }

                    if (ratio < ratioElapsed)
                    {
                        colType = ColType.COL_WALL;
                        colWall = Wall.UP_WALL;
                                                
                        ratioElapsed = ratio;
                        tempElapsed = ratio * restElapsed;
                        colPoint = c;

                        newDir = ballNewPos - ballOldPos;
                        newDir.Z = -newDir.Z;
                        newDir.Normalize();
                    }
                }

                // Down wall                
                if (ballOldPos.Z <= halfTableW - ballRadius
                    && ballNewPos.Z > halfTableW - ballRadius)
                {
                    float ratio = (halfTableW - ballRadius - ballOldPos.Z)
                        / (ballNewPos.Z - ballOldPos.Z);

                    c = ballOldPos + ratio * ballRelPos;
                    while (c.Z > halfTableW - ballRadius - 0.001 && ratio > 0)
                    {
                        if (ratio < 1e-3)
                            ratio = 0;
                        else
                            ratio -= 1e-3F;
                        c = ballOldPos + ratio * ballRelPos;
                    }

                    if (ratio < ratioElapsed)
                    {
                        colType = ColType.COL_WALL;
                        colWall = Wall.DOWN_WALL;
                        
                        ratioElapsed = ratio;
                        tempElapsed = ratio * restElapsed;
                        colPoint = c;

                        newDir = ballNewPos - ballOldPos;
                        newDir.Z = -newDir.Z;
                        newDir.Normalize();
                    }
                }

                // Collide ball with man
                for (int i = 0; i < FoosGame.BLACKMEN_NUM + FoosGame.YELLOWMEN_NUM; i++)
                //for (int i = 1; i < 2; i++)
                {                                 
                    
                    // Distance limit between ball and man's collision ball
                    float dist_s = ballRadius + oBMen[i]._colSphere.Radius;

                    // man's old position
                    Vector3 manOldPos = oBMen[i]._prevPos;
                    Vector3 manRelPos = oBMen[i]._curPos - manOldPos;                   

                    // ****************************************************************************
                    // Following calculation depends on looking the 
                    // man'scollision ball as a static coordinate system. 
                    // ****************************************************************************

                    //
                    // First, calculate shortest distance to decide collision or not
                    //

                    // ball's relative position to man
                    Vector3 o = ballOldPos - manOldPos;
                   // if (o.X >= FoosGame.ROD_TO_ROD || o.X <= -FoosGame.ROD_TO_ROD
                     //   || o.Z >= FoosGame.ROD_TO_ROD || o.Z <= -FoosGame.ROD_TO_ROD)
                       // continue;

                    float ol = o.Length();                  
                    Debug.Assert(ol >= dist_s); // Hopefully:)

                    //if (ol >= FoosGame.ROD_TO_ROD)
                      //  continue;

                    // ball's relative displacement to man
                    Vector3 u = ballRelPos - manRelPos;

                    // ball's relative end position to man
                    Vector3 e = o + u;

                    // Now get the length of relative displacement and its normal
                    float ul = u.Length();
                    u.Normalize();
                   

                    float t = - Vector3.Dot(o, u);
                    Vector3 s;  // s stores the shortest vector from origion to ball
                    if (t >= ul)                    
                        s = e;                    
                    else if (t <= 0)                   
                        s = o;                    
                    else                    
                        s = o + t * u;
                    

                    // If we collide
                    float sl = s.Length();                   
                    if (sl < dist_s)
                    {
                        // Find the collision point
                        float ol2 = o.LengthSquared();
                        //float dist_s2 = dist_s * dist_s;                                               

                        float delta = (float)Math.Sqrt((double)(t * t + dist_s * dist_s - ol2));

                        float t1 = t - delta;
                       // if (t1 < 1e-5F)
                       //     t1 = 0;
                        Debug.Assert(t1 <= ul && t1 >= 0);
                       
                        // Now we got t1
                        float ratio = t1 / ul;
                                                                   
                       

                        // c is collision vector in man's coordinate system
                        // cu is the collision normal vector   
                                               
                        c = o + t1 * u;
                        while (c.Length() < dist_s + 0.001 && t1 > 0)
                        {
                            if (t1 < 1e-3F)
                                t1 = 0;
                            else
                                t1 -= 1e-3F;
                            c = o + t1 * u;
                        }
                        ratio = t1 / ul;

                        if (ratio < ratioElapsed)
                        {
                            colType = ColType.COL_MAN;

                            colObj = oBMen[i]; 
                            ratioElapsed = ratio;
                            tempElapsed = ratio * restElapsed;
                            colPoint = c;

                            Vector3 cu = c;
                            float cul = c.Length();
                            cu.Normalize();

                            Vector3 c2 = Vector3.Dot(o, cu) * cu - o;
                            newDir = c2 + c2 + o - c;
                            if (newDir.Length() < 1e-6)
                                newDir = Vector3.Zero;
                            else
                                newDir.Normalize();                            
                        }
                    }                                        
                }

                // Update all men to ratio                               
                for (int i = 0; i < FoosGame.BLACKMEN_NUM + FoosGame.YELLOWMEN_NUM; i++)               
                    oBMen[i]._prevPos = oBMen[i]._prevPos + ratioElapsed * (oBMen[i]._curPos - oBMen[i]._prevPos);                

                restElapsed -= tempElapsed;
                
                if (colType == ColType.COL_NONE)
                {
                    ball.position = ballOldPos + ballRelPos;
                    ball.Speed = 0.999F * ball.Speed;
                }
                else if (colType == ColType.COL_MAN)
                {
                    FoosMan fm = (FoosMan)colObj;                   

                    //fm._curPos = fm._prevPos;
                    //fm._rot_z = fm._rot_z_old + ratioElapsed * (fm._rot_z - fm._rot_z_old);
                    //fm._pos.Z = fm._pos_z_old + ratioElapsed * (fm._pos.Z - fm._pos_z_old);

#if false
                    fm._curPos = fm._prevPos;
                    

                    fm._rod.zRotation = fm._rod.zRotationOld + 
                        ratioElapsed * (fm._rod.zRotation - fm._rod.zRotationOld);
#else   

                    if (fm._rod.PlayerIndex == 1)
                    {
                        for (int rr = 0; rr < FoosGame.BLACKROD_NUM; rr++)
                        {
                            Rod rodrod = (Rod)allObjs[FoosGame.OI_BLACKROD + rr];
                            rodrod.zRotation = rodrod.zRotationOld +
                                ratioElapsed * (rodrod.zRotation - rodrod.zRotationOld);
                        }

                        
                        for (int mm = 0; mm < FoosGame.BLACKMEN_NUM; mm++)
                        {
                            FoosMan manman = (FoosMan)allObjs[FoosGame.OI_BLACKMEN + mm];
                            manman._curPos = manman._prevPos;
                        }
                        
                    }
                    else if (fm._rod.PlayerIndex == 2)
                    {
                        for (int rr = 0; rr < FoosGame.YELLOWROD_NUM; rr++)
                        {
                            Rod rodrod = (Rod)allObjs[FoosGame.OI_YELLOWROD + rr];
                            rodrod.zRotation = rodrod.zRotationOld +
                                ratioElapsed * (rodrod.zRotation - rodrod.zRotationOld);
                        }
                        
                        for (int mm = 0; mm < FoosGame.YELLOWMEN_NUM; mm++)
                        {
                            FoosMan manman = (FoosMan)allObjs[FoosGame.OI_YELLOWMEN + mm];
                            manman._curPos = manman._prevPos;
                        }
                    }
                    else
                        Debug.Assert(false);
#endif

                    fm._rod.positionZ = fm._rod.zPosOld + 
                        ratioElapsed * (fm._rod.positionZ - fm._rod.zPosOld);

                    ball.position = fm._prevPos + colPoint;                    
                  //  ball.Speed = ball.Speed.Length() * 0.5F * newDir + fm._speed;
                    ball.Speed = (ball.Speed - fm._speed).Length() * 0.3F * newDir + 0.7F * fm._speed;
                    ball.SpeedY = 0;

                    ll = (ball.position - fm._prevPos).Length();
                    Debug.Assert(ll >= 4.75);

                    if (ratioElapsed > 1e-3 && fm.Collided == 0)
                    {
                        fm.Collided = 1;
                        game.SoundBank.PlayCue("collision");
                    }
                }
                else if (colType == ColType.COL_WALL)
                {                    
                    ball.position = colPoint;
                    ball.Speed = ball.Speed.Length() * newDir * 0.8F;

                     if (colWall == Wall.NO_INDEX_WALL)
                            game.SoundBank.PlayCue("collision");
                    else if (ratioElapsed > 1e-3 && _collidedWall[(int)colWall] == 0)
                        {
                            _collidedWall[(int)colWall] = 1;
                            game.SoundBank.PlayCue("collision");
                        }
                    

                }
                
                if (ball.positionY > FoosGame.BALL_RADIUS)
                    ball.positionY = FoosGame.BALL_RADIUS;
               
                colLast = colType;

                
                /*
                if (tempElapsed < 1e-3)
                {
                    contiZero++;
                    if (contiZero == 3)
                    {                        
                        break;
                    }
                }
                else
                    contiZero = 0;*/
                if (contiZero++ > 10)
                    break;                
                
            }

            for (int i = 0; i < FoosGame.BLACKMEN_NUM + FoosGame.YELLOWMEN_NUM; i++)
            {
                oBMen[i]._pos.Z = oBMen[i]._rod.positionZ + oBMen[i]._relativeToRod.Z;
                oBMen[i]._rot_z = oBMen[i]._rod.zRotation;
            }            
        }
    }
}

